[libcxx-commits] [PATCH] D99373: [SystemZ][z/OS] tune down warning about unused parameter on nanosleep()

Zibi Sarbino via Phabricator via libcxx-commits libcxx-commits at lists.llvm.org
Thu Apr 1 07:53:01 PDT 2021


zibi updated this revision to Diff 334696.
zibi marked 2 inline comments as done.
zibi added a comment.

addressing comments


Repository:
  rG LLVM Github Monorepo

CHANGES SINCE LAST ACTION
  https://reviews.llvm.org/D99373/new/

https://reviews.llvm.org/D99373

Files:
  libcxx/include/__support/ibm/nanosleep.h


Index: libcxx/include/__support/ibm/nanosleep.h
===================================================================
--- libcxx/include/__support/ibm/nanosleep.h
+++ libcxx/include/__support/ibm/nanosleep.h
@@ -12,27 +12,44 @@
 
 #include <unistd.h>
 
-inline int nanosleep(const struct timespec* req, struct timespec* rem)
-{
-   // The nanosleep() function is not available on z/OS. Therefore, we will call
-   // sleep() to sleep for whole seconds and usleep() to sleep for any remaining
-   // fraction of a second. Any remaining nanoseconds will round up to the next
-   // microsecond.
-
-   useconds_t __micro_sec = (rem->tv_nsec + 999) / 1000;
-   if (__micro_sec > 999999)
-   {
-     ++rem->tv_sec;
-     __micro_sec -= 1000000;
-   }
-   while (rem->tv_sec)
-      rem->tv_sec = sleep(rem->tv_sec);
-   if (__micro_sec) {
-     rem->tv_nsec = __micro_sec * 1000;
-     return usleep(__micro_sec);
-   }
-   rem->tv_nsec = 0;
-   return 0;
+inline int nanosleep(const struct timespec* __req, struct timespec* __rem) {
+  // The nanosleep() function is not available on z/OS. Therefore, we will call
+  // sleep() to sleep for whole seconds and usleep() to sleep for any remaining
+  // fraction of a second. Any remaining nanoseconds will round up to the next
+  // microsecond.
+  if (__req->tv_sec < 0 || __req->tv_nsec < 0 || __req->tv_nsec > 999999999) {
+    errno = EINVAL;
+    return -1;
+  }
+  useconds_t __micro_sec =
+      static_cast<useconds_t>((__req->tv_nsec + 999) / 1000);
+  time_t __sec = __req->tv_sec;
+  if (__micro_sec > 999999) {
+    ++__sec;
+    __micro_sec -= 1000000;
+  }
+  __sec = sleep(static_cast<unsigned int>(__sec));
+  if (__sec) {
+    if (__rem) {
+      __rem->tv_sec = __sec;
+      // nanoseconds can be a large number, 999999999 at the max
+      __rem->tv_nsec = __micro_sec * 1000;
+    }
+    errno = EINTR;
+    return -1;
+  }
+  if (__micro_sec) {
+    int __rt = usleep(__micro_sec);
+    if (__rt != 0 && __rem) {
+      __rem->tv_sec = 0;
+      // nanoseconds can be a large number, 999999999 at the max
+      __rem->tv_nsec = __micro_sec * 1000;
+      // errno is already set
+      return -1;
+    }
+    return __rt;
+  }
+  return 0;
 }
 
 #endif // _LIBCPP_SUPPORT_IBM_NANOSLEEP_H


-------------- next part --------------
A non-text attachment was scrubbed...
Name: D99373.334696.patch
Type: text/x-patch
Size: 2253 bytes
Desc: not available
URL: <http://lists.llvm.org/pipermail/libcxx-commits/attachments/20210401/c717bcd4/attachment-0001.bin>


More information about the libcxx-commits mailing list